// $Id$

/***************************************************************************
*   Copyright (C) 2004 by Thomas Fischer                                  *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qstringlist.h>

#include <bibtexentry.h>
#include <bibtexfile.h>
#include <bibtexencoder.h>

BibTeXEntry::BibTeXEntry( const ElementType elementType, const QString &id, QObject *parent, const char *name )
        : BibTeXElement( parent, name ), m_elementType( elementType ), m_id( id )
{}

BibTeXEntry::BibTeXEntry( BibTeXEntry *other )
{
    m_elementType = other->m_elementType;
    m_id = other->m_id;
    m_fields = other->m_fields;
}

BibTeXEntry::~BibTeXEntry()
{}

void BibTeXEntry::setElementType( const ElementType elementType )
{
    m_elementType = elementType;
}

BibTeXEntry::ElementType BibTeXEntry::elementType()
{
    return m_elementType;
}

void BibTeXEntry::setId( const QString& id )
{
    m_id = id;
}

QString BibTeXEntry::id()
{
    return m_id;
}

QString BibTeXEntry::toBibTeXString( bool htmlHighlighting )
{
    QString result( "@" );

    if ( htmlHighlighting )
        result.append( "<font color=\"#000099\">" );
    result.append( elementTypeToString( m_elementType ) );
    if ( htmlHighlighting )
        result.append( "</font>" );

    result.append( " { " );

    if ( htmlHighlighting )
        result.append( "<font color=\"#000099\"><b>" );
    result.append( m_id );
    if ( htmlHighlighting )
        result.append( "</b></font>" );

    for ( QValueList<Field*>::iterator it = m_fields.begin(); it != m_fields.end(); it++ )
    {
        QString text = BibTeXEncoder::currentBibTeXEncoder() ->encodeSpecialized( ( *it ) ->value, ( *it ) ->fieldType );

        BibTeXFile::StringProtection sp = BibTeXFile::spQuote;
        if ( ( ( *it ) ->fieldType == ftTitle ) || ( ( *it ) ->fieldType == ftBookTitle ) )
            sp = BibTeXFile::spBoth;
        else if ( ( ( *it ) ->fieldType == ftMonth ) || ( ( *it ) ->fieldType == ftYear ) )
            sp = BibTeXFile::spNone;

        result.append( ",\n\t" );
        if ( htmlHighlighting )
            result.append( "<font color=\"#990000\">" );
        result.append( ( *it ) ->key );
        if ( htmlHighlighting )
            result.append( "</font>" );
        result.append( " = " );
        switch ( sp )
        {
        case BibTeXFile::spNone:
            break;
        case BibTeXFile::spParanthesis:
            result.append( "{" );
            break;
        case BibTeXFile::spQuote:
            result.append( "\"" );
            break;
        case BibTeXFile::spBoth:
            result.append( "\"{" );
            break;
        }

        if ( htmlHighlighting )
            result.append( "<font color=\"#006633\">" );
        result.append( text );
        if ( htmlHighlighting )
            result.append( "</font>" );

        switch ( sp )
        {
        case BibTeXFile::spNone:
            break;
        case BibTeXFile::spParanthesis:
            result.append( "}" );
            break;
        case BibTeXFile::spQuote:
            result.append( "\"" );
            break;
        case BibTeXFile::spBoth:
            result.append( "}\"" );
            break;
        }
    }

    return result.append( "\n}" );
}

bool BibTeXEntry::containsPattern( QString& pattern, bool caseSensitive )
{
    bool result = false;

    for ( QValueList<Field*>::iterator it = m_fields.begin(); !result && ( it != m_fields.end() ); it++ )
        result = ( *it ) ->value.find( pattern, 0, caseSensitive ) != -1;

    return result;
}

BibTeXEntry::Field* BibTeXEntry::field( const FieldType fieldType )
{
    Field * result = NULL;

    for ( QValueList<Field*>::iterator it = m_fields.begin(); ( it != m_fields.end() ) && ( result == NULL ); it++ )
        if ( ( *it ) ->fieldType == fieldType ) result = *it;

    return result;
}

BibTeXEntry::Field * BibTeXEntry::field( const QString& key )
{
    Field * result = NULL;

    for ( QValueList<Field*>::iterator it = m_fields.begin(); ( it != m_fields.end() ) && ( result == NULL ); it++ )
        if ( ( *it ) ->key.lower() == key.lower() )
            result = *it;

    return result;
}

bool BibTeXEntry::setField( const QString& key, const QString &value )
{
    FieldType fieldType = fieldTypeFromString( key );
    if ( fieldType == ftUnknown )
    {
        bool found = FALSE;
        QValueList<Field*>::iterator it;
        QString decValue = BibTeXEncoder::currentBibTeXEncoder() ->decode( value );

        for ( it = m_fields.begin(); it != m_fields.end() ; it++ )
            if ( ( found = ( *it ) ->key.lower() == key.lower() ) )
                break;

        if ( found )
            ( * it ) ->value = value;
        else
        {
            Field *newField = new Field;
            newField->fieldType = fieldType;
            newField->key = key;
            newField->value = value;
            m_fields.append( newField );
        }
    }
    else
        return setField( fieldType, value );

    return TRUE;
}

bool BibTeXEntry::setField( const FieldType fieldType, const QString &value )
{
    QValueList<Field*>::iterator it;
    bool found = FALSE;
    QString decValue = BibTeXEncoder::currentBibTeXEncoder() ->decodeSpecialized( value, fieldType );
    //qDebug( "value=%s   decValue=%s", value.latin1(), decValue.latin1() );

    for ( it = m_fields.begin(); it != m_fields.end(); it++ )
        if ( ( found = ( *it ) ->fieldType == fieldType ) )
            break;

    if ( found )
        ( * it ) ->value = decValue;
    else
    {
        Field *newField = new Field;
        newField->fieldType = fieldType;
        newField->key = fieldTypeToString( fieldType );
        newField->value = decValue;
        m_fields.append( newField );
    }

    return TRUE;
}

bool BibTeXEntry::deleteField( const QString &key )
{
    QValueList<Field*>::iterator it;
    bool found = FALSE;

    for ( it = m_fields.begin(); it != m_fields.end(); it++ )
        if ( ( found = ( *it ) ->key.lower() == key.lower() ) )
            break;

    if ( found )
        m_fields.remove( it );

    return found;
}

bool BibTeXEntry::deleteField( const FieldType fieldType )
{
    QValueList<Field*>::iterator it;
    bool found = FALSE;

    for ( it = m_fields.begin(); it != m_fields.end(); it++ )
        if ( ( found = ( *it ) ->fieldType == fieldType ) )
            break;

    if ( found )
        m_fields.remove( it );

    return found;
}

BibTeXEntry::FieldType BibTeXEntry::fieldTypeFromString( const QString &fieldTypeString )
{
    QString fieldTypeStringLower = fieldTypeString.lower();

    if ( fieldTypeStringLower.compare( "abstract" ) == 0 )
        return ftAbstract;
    else if ( fieldTypeStringLower.compare( "address" ) == 0 )
        return ftAddress;
    else if ( fieldTypeStringLower.compare( "annote" ) == 0 )
        return ftAnnote;
    else if ( fieldTypeStringLower.compare( "author" ) == 0 )
        return ftAuthor;
    else if ( fieldTypeStringLower.compare( "booktitle" ) == 0 )
        return ftBookTitle;
    else if ( fieldTypeStringLower.compare( "chapter" ) == 0 )
        return ftChapter;
    else if ( fieldTypeStringLower.compare( "crossref" ) == 0 )
        return ftCrossRef;
    else if ( fieldTypeStringLower.compare( "edition" ) == 0 )
        return ftEdition;
    else if ( fieldTypeStringLower.compare( "editor" ) == 0 )
        return ftEditor;
    else if ( fieldTypeStringLower.compare( "howpublished" ) == 0 )
        return ftHowPublished;
    else if ( fieldTypeStringLower.compare( "institution" ) == 0 )
        return ftInstitution;
    else if ( fieldTypeStringLower.compare( "isbn" ) == 0 )
        return ftISBN;
    else if ( fieldTypeStringLower.compare( "issn" ) == 0 )
        return ftISSN;
    else if ( fieldTypeStringLower.compare( "journal" ) == 0 )
        return ftJournal;
    else if ( fieldTypeStringLower.compare( "key" ) == 0 )
        return ftKey;
    else if ( fieldTypeStringLower.compare( "month" ) == 0 )
        return ftMonth;
    else if ( fieldTypeStringLower.compare( "note" ) == 0 )
        return ftNote;
    else if ( fieldTypeStringLower.compare( "number" ) == 0 )
        return ftNumber;
    else if ( fieldTypeStringLower.compare( "organization" ) == 0 )
        return ftOrganization;
    else if ( fieldTypeStringLower.compare( "pages" ) == 0 )
        return ftPages;
    else if ( fieldTypeStringLower.compare( "publisher" ) == 0 )
        return ftPublisher;
    else if ( fieldTypeStringLower.compare( "series" ) == 0 )
        return ftSeries;
    else if ( fieldTypeStringLower.compare( "school" ) == 0 )
        return ftSchool;
    else if ( fieldTypeStringLower.compare( "title" ) == 0 )
        return ftTitle;
    else if ( fieldTypeStringLower.compare( "type" ) == 0 )
        return ftType;
    else if ( fieldTypeStringLower.compare( "url" ) == 0 )
        return ftURL;
    else if ( fieldTypeStringLower.compare( "volume" ) == 0 )
        return ftVolume;
    else if ( fieldTypeStringLower.compare( "year" ) == 0 )
        return ftYear;
    else
        return ftUnknown;
}

QString BibTeXEntry::fieldTypeToString( const FieldType fieldType )
{
    switch ( fieldType )
    {
    case ftAbstract:
        return QString( "abstract" );
    case ftAddress:
        return QString( "address" );
    case ftAnnote:
        return QString( "annote" );
    case ftAuthor:
        return QString( "author" );
    case ftBookTitle:
        return QString( "booktitle" );
    case ftChapter:
        return QString( "chapter" );
    case ftCrossRef:
        return QString( "crossref" );
    case ftEdition:
        return QString( "edition" );
    case ftEditor:
        return QString( "editor" );
    case ftHowPublished:
        return QString( "howpublished" );
    case ftInstitution:
        return QString( "institution" );
    case ftISBN:
        return QString( "isbn" );
    case ftISSN:
        return QString( "issn" );
    case ftJournal:
        return QString( "journal" );
    case ftKey:
        return QString( "key" );
    case ftMonth:
        return QString( "month" );
    case ftNote:
        return QString( "note" );
    case ftNumber:
        return QString( "number" );
    case ftOrganization:
        return QString( "organization" );
    case ftPages:
        return QString( "pages" );
    case ftPublisher:
        return QString( "publisher" );
    case ftSeries:
        return QString( "series" );
    case ftSchool:
        return QString( "school" );
    case ftTitle:
        return QString( "title" );
    case ftType:
        return QString( "type" );
    case ftURL:
        return QString( "url" );
    case ftVolume:
        return QString( "volume" );
    case ftYear:
        return QString( "year" );
    default:
        return QString( "unknown" );
    }
}

BibTeXEntry::FieldRequireStatus BibTeXEntry::getRequireStatus( BibTeXElement::ElementType elementType, BibTeXEntry::FieldType fieldType )
{
    switch ( elementType )
    {
    case etArticle:
        switch ( fieldType )
        {
        case ftAuthor:
        case ftTitle:
        case ftJournal:
        case ftYear:
            return frsRequired;
        case ftVolume:
        case ftMonth:
        case ftNumber:
        case ftPages:
        case ftNote:
        case ftKey:
        case ftISSN:
            return frsOptional;
        default: return frsIgnored;
        }
    case etBook:
        switch ( fieldType )
        {
        case ftAuthor:
        case ftEditor:
        case ftTitle:
        case ftPublisher:
        case ftYear:
            return frsRequired;
        case ftVolume:
        case ftSeries:
        case ftAddress:
        case ftEdition:
        case ftMonth:
        case ftNote:
        case ftKey:
        case ftISBN:
            return frsOptional;
        default: return frsIgnored;
        }
    case etBooklet:
        switch ( fieldType )
        {
        case ftTitle:
            return frsRequired;
        case ftAuthor:
        case ftHowPublished:
        case ftAddress:
        case ftMonth:
        case ftYear:
        case ftNote:
        case ftKey:
        case ftISBN:
            return frsOptional;
        default: return frsIgnored;
        }
    case etInBook:
        switch ( fieldType )
        {
        case ftAuthor:
        case ftEditor:
        case ftTitle:
        case ftChapter:
        case ftPages:
        case ftPublisher:
        case ftYear:
            return frsRequired;
        case ftVolume:
        case ftSeries:
        case ftAddress:
        case ftEdition:
        case ftMonth:
        case ftNote:
        case ftKey:
        case ftISBN:
            return frsOptional;
        default: return frsIgnored;
        }
    case etInCollection:
        switch ( fieldType )
        {
        case ftAuthor:
        case ftTitle:
        case ftBookTitle:
        case ftPublisher:
        case ftYear:
            return frsRequired;
        case ftEditor:
        case ftPages:
        case ftOrganization:
        case ftAddress:
        case ftMonth:
        case ftNote:
        case ftKey:
        case ftType:
        case ftISBN:
            return frsOptional;
        default: return frsIgnored;
        }
    case etConference:
    case etInProceedings:
        switch ( fieldType )
        {
        case ftAuthor:
        case ftTitle:
        case ftYear:
            return frsRequired;
        case ftPages:
        case ftBookTitle:
        case ftEditor:
        case ftVolume:
        case ftNumber:
        case ftSeries:
        case ftType:
        case ftChapter:
        case ftAddress:
        case ftEdition:
        case ftMonth:
        case ftNote:
        case ftCrossRef:
        case ftPublisher:
        case ftISBN:
            return frsOptional;
        default: return frsIgnored;
        }
    case etManual:
        switch ( fieldType )
        {
        case ftTitle:
            return frsRequired;
        case ftAuthor:
        case ftOrganization:
        case ftAddress:
        case ftEdition:
        case ftMonth:
        case ftYear:
        case ftNote:
        case ftISBN:
            return frsOptional;
        default: return frsIgnored;
        }
    case etMastersThesis:
        switch ( fieldType )
        {
        case ftAuthor:
        case ftTitle:
        case ftSchool:
        case ftYear:
            return frsRequired;
        case ftAddress:
        case ftMonth:
        case ftNote:
        case ftKey:
            return frsOptional;
        default: return frsIgnored;
        }
    case etMisc:
        switch ( fieldType )
        {
        case ftAuthor:
        case ftTitle:
        case ftHowPublished:
        case ftMonth:
        case ftYear:
        case ftNote:
        case ftKey:
            return frsOptional;
        default: return frsIgnored;
        }
    case etPhDThesis:
        switch ( fieldType )
        {
        case ftAuthor:
        case ftTitle:
        case ftSchool:
        case ftYear:
            return frsRequired;
        case ftAddress:
        case ftMonth:
        case ftNote:
        case ftKey:
        case ftISBN:
            return frsOptional;
        default: return frsIgnored;
        }
    case etCollection:
    case etProceedings:
        switch ( fieldType )
        {
        case ftTitle:
        case ftYear:
            return frsRequired;
        case ftEditor:
        case ftPublisher:
        case ftOrganization:
        case ftAddress:
        case ftMonth:
        case ftNote:
        case ftKey:
        case ftSeries:
        case ftBookTitle:
        case ftISBN:
            return frsOptional;
        default: return frsIgnored;
        }
    case etTechReport:
        switch ( fieldType )
        {
        case ftAuthor:
        case ftTitle:
        case ftInstitution:
        case ftYear:
            return frsRequired;
        case ftType:
        case ftNumber:
        case ftAddress:
        case ftMonth:
        case ftNote:
        case ftKey:
            return frsOptional;
        default: return frsIgnored;
        }
    case etUnpublished:
        switch ( fieldType )
        {
        case ftAuthor:
        case ftTitle:
        case ftNote:
            return frsRequired;
        case ftMonth:
        case ftYear:
        case ftKey:
            return frsOptional;
        default: return frsIgnored;
        }
    default:
        return frsOptional;
    }
}

int BibTeXEntry::getFieldCount()
{
    return m_fields.count();
}

QValueList<BibTeXEntry::Field*>::iterator BibTeXEntry::begin()
{
    return m_fields.begin();
}

QValueList<BibTeXEntry::Field*>::iterator BibTeXEntry::end()
{
    return m_fields.end();
}

#include "bibtexentry.moc"
